/*
 * FSVideoData.java
 * Transform
 *
 * Copyright (c) 2001-2008 Flagstone Software Ltd. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *  * Neither the name of Flagstone Software Ltd. nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.flagstone.transform;

/**
 * The FSVideoMetaData class is used to store information on how the video
 * stream should be displayed.
 * 
 * Although meta-data can be found in all flash Video files there is no
 * documentation published by Adobe that describes the data structure. As a
 * result the information is decoded as a simple block of binary data.
 */
public final class FSVideoMetaData extends FSVideoObject
{
	private byte[] data;

	FSVideoMetaData(FSCoder coder)
	{
		super(FSVideoObject.MetaData, 0);
		decode(coder);
	}

	/**
	 * Get the encoded meta data that decribes how the video stream should be
	 * played.
	 * 
	 * @return an array of bytes containing the encoded meta-data.
	 */
	public byte[] getData()
	{
		return data;
	}

	/**
	 * Sets the encoded meta data that decribes how the video stream should be
	 * played.
	 * 
	 * @param data
	 *            an array of bytes containing the encoded meta-data.
	 */
	public void setData(byte[] data)
	{
		this.data = data;
	}

	public Object clone()
	{
		FSVideoMetaData anObject = (FSVideoMetaData) super.clone();

		return anObject;
	}

	public boolean equals(Object anObject)
	{
		boolean result = false;

		if (super.equals(anObject))
		{
			FSVideoMetaData typedObject = (FSVideoMetaData) anObject;

			if (data != null)
				result = result && Transform.equals(data, typedObject.data);
			else
				result = result && typedObject.data == null;
		}
		return result;
	}

	public void appendDescription(StringBuffer buffer, int depth)
	{
		buffer.append(name());

		if (depth > 0)
		{
			buffer.append(": { ");
			Transform.append(buffer, "data", "<data>");
			buffer.append("}");
		}
	}

	int length(FSCoder coder)
	{
		super.length(coder);

		length += data.length;

		return length;
	}

	void encode(FSCoder coder)
	{
		super.encode(coder);
		coder.writeBytes(data);
	}

	void decode(FSCoder coder)
	{
		super.decode(coder);
		data = new byte[length];
		coder.readBytes(data);
	}
}